# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np


def train_cv(self, X, T, k):
    """Model structure selection with cross-validation.

    Trains ELM, cross-validates model and sets an optimal validated solution.

    Args:
        self (ELM): ELM object that calls `train_v()`
        X (matrix): training set inputs
        T (matrix): training set outputs
        k (int): number of parts to split the dataset into, k-2 parts are used for training and 2 parts are
            left out: 1 for validation and 1 for test; repeated k times until all parts have been left out for
            validation and test, and results averaged over these k repetitions.

    Returns:
        err_t (double): error for the optimal model, computed in the 'cross-testing' manner on data part
        which is not used for training or validation
    """
    N = X.shape[0]
    L = self.nnet.L
    c = self.nnet.outputs

    idxk = []
    for i in range(k):
        idxk.append(np.arange(N)[i::k])

    datak = []
    for i in range(k):
        items = [(i+j) % k for j in range(k)]
        idx_tr = np.hstack([idxk[j] for j in items[:-2]])
        idx_vl = idxk[items[-2]]
        idx_ts = idxk[items[-1]]
        Xtr = X[idx_tr]
        Ttr = T[idx_tr]
        Xvl = X[idx_vl]
        Tvl = T[idx_vl]
        Xts = X[idx_ts]
        Tts = T[idx_ts]
        self.nnet.reset()
        self.nnet.add_batch(Xtr, Ttr)
        HH, HT = self.nnet.get_corr()
        Hvl = self.nnet._project(Xvl)
        Hts = self.nnet._project(Xts)
        rank, L = self._ranking(Hvl.shape[1], Hvl, Tvl)
        datak.append((HH, HT, Hvl, Tvl, Hts, Tts, rank))

    e = np.ones((L+1,)) * -1  # errors for all numbers of neurons

    err = 0
    for HH, HT, Hvl, Tvl, _, _, _ in datak:
        B = self.nnet.solve_corr(HH, HT)
        Yvl = np.dot(Hvl, B)
        err += self._error(Tvl, Yvl) / k
    penalty = err * 0.01 / L  # penalty is 1% of error at max(L)
    e[L] = err + L * penalty

    # MYOPT function
    # [iA  iB  iC  iD  iE] interval points,
    # halve the interval each time

    # initialize intervals
    iA = 3
    iE = L
    l = iE - iA
    iB = iA + l/4
    iC = iA + l/2
    iD = iA + 3*l/4

    # TODO: tell about single-letter matrix notations in the whole code
    l = 1000  # run the while loop at least once
    while l > 2:
        # calculate errors at points
        for idx in [iA, iB, iC, iD, iE]:
            if e[idx] == -1:  # skip already calculated errors
                err = 0
                for HH, HT, Hvl, Tvl, _, _, rank in datak:
                    rank1 = rank[:idx]
                    HH1 = HH[rank1, :][:, rank1]
                    HT1 = HT[rank1, :]
                    B = self.nnet.solve_corr(HH1, HT1)
                    Yvl = np.dot(Hvl[:, rank1], B)
                    err += self._error(Tvl, Yvl) / k
                e[idx] = err + idx * penalty

        m = min(e[iA], e[iB], e[iC], e[iD], e[iE])  # find minimum element

        # halve the search interval
        if m in (e[iA], e[iB]):
            iE = iC
            iC = iB
        elif m in (e[iD], e[iE]):
            iA = iC
            iC = iD
        else:
            iA = iB
            iE = iD
        l = iE - iA
        iB = iA + l/4
        iD = iA + (3*l)/4

    k_opt = [n1 for n1 in [iA, iB, iC, iD, iE] if e[n1] == m][0]  # find minimum index
    best_L = rank[:k_opt]

    # get test error
    err_t = 0
    for HH, HT, _, _, Hts, Tts, _ in datak:
        B = self.nnet.solve_corr(HH, HT)
        Yts = np.dot(Hts, B)
        err_t += self._error(Tts, Yts) / k

    self.nnet._prune(best_L)
    self.nnet.add_batch(X, T)
    self.nnet.solve()
#    print "%d of %d neurons selected with a Cross-Validation" % (len(best_nn), nn)
#    print "the Cross-Validation test error is %f" % err_ts
#    if len(best_nn) > nn*0.9:
#        print "Hint: try re-training with more hidden neurons"
    return err_t





















# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 14:12:41 2014

@author: akusok
"""

import os
import tempfile
from unittest import TestCase

import numpy as np

from hpelm import ELM


class TestCorrectness(TestCase):
    def test_NonNumpyInputs_RaiseError(self):
        X = np.array([['1', '2'], ['3', '4'], ['5', '6']])
        T = np.array([[1], [2], [3]])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_NonNumpyTargets_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([['a'], ['b'], ['c']])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_OneDimensionInputs_RunsCorrectly(self):
        X = np.array([1, 2, 3])
        T = np.array([[1], [2], [3]])
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_OneDimensionTargets_RunsCorrectly(self):
        X = np.array([1, 2, 3])
        T = np.array([1, 2, 3])
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_WrongDimensionalityInputs_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[1], [2], [3]])
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_WrongDimensionalityTargets_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[1], [2], [3]])
        elm = ELM(1, 2)
        elm.add_neurons(1, "lin")
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_ZeroInputs_RunsCorrectly(self):
        X = np.array([[0, 0], [0, 0], [0, 0]])
        T = np.array([1, 2, 3])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_OneDimensionTargets2_RunsCorrectly(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[0], [0], [0]])
        elm = ELM(2, 1)
        elm.add_neurons(1, "lin")
        elm.train(X, T)

    def test_TrainWithoutNeurons_RaiseError(self):
        X = np.array([1, 2, 3])
        T = np.array([1, 2, 3])
        elm = ELM(1, 1)
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_DifferentNumberOfSamples_RaiseError(self):
        X = np.array([[1, 2], [3, 4], [5, 6]])
        T = np.array([[1], [2]])
        elm = ELM(2, 1)
        self.assertRaises(AssertionError, elm.train, X, T)

    def test_LinearNeurons_MoreThanInputs_Truncated(self):
        elm = ELM(2, 1)
        elm.add_neurons(3, "lin")
        self.assertEqual(2, elm.nnet.get_neurons()[0][0])

    def test_LinearNeurons_DefaultMatrix_Identity(self):
        elm = ELM(4, 1)
        elm.add_neurons(3, "lin")
        np.testing.assert_array_almost_equal(np.eye(4, 3), elm.nnet.get_neurons()[0][2])

    def test_SLFN_AddLinearNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        self.assertEquals("lin", elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddSigmoidalNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "sigm")
        self.assertEquals("sigm", elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddTanhNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "tanh")
        self.assertEquals("tanh", elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddRbfL1Neurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "rbf_l1")
        self.assertEquals("rbf_l1", elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddRbfL2Neurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "rbf_l2")
        self.assertEquals("rbf_l2", elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddRbfLinfNeurons_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "rbf_linf")
        self.assertEquals("rbf_linf", elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddUfuncNeurons_GotThem(self):
        elm = ELM(1, 1)
        func = np.frompyfunc(lambda a: a + 1, 1, 1)
        elm.add_neurons(1, func)
        self.assertIs(func, elm.nnet.get_neurons()[0][1])

    def test_SLFN_AddTwoNeuronTypes_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.add_neurons(1, "sigm")
        self.assertEquals(2, len(elm.nnet.get_neurons()))
        ntypes = [nr[1] for nr in elm.nnet.get_neurons()]
        self.assertIn("lin", ntypes)
        self.assertIn("sigm", ntypes)

    def test_SLFN_AddNeuronsTwice_GotThem(self):
        elm = ELM(1, 1)
        elm.add_neurons(1, "lin")
        elm.add_neurons(1, "lin")
        self.assertEquals(1, len(elm.nnet.get_neurons()))
        self.assertEquals(2, elm.nnet.get_neurons()[0][0])

    def test_AddNeurons_InitBias_BiasInModel(self):
        elm = ELM(1, 1)
        bias = np.array([1, 2, 3])
        elm.add_neurons(3, "sigm", None, bias)
        np.testing.assert_array_almost_equal(bias, elm.nnet.get_neurons()[0][3])

    def test_AddNeurons_InitW_WInModel(self):
        elm = ELM(2, 1)
        W = np.array([[1, 2, 3], [4, 5, 6]])
        elm.add_neurons(3, "sigm", W, None)
        np.testing.assert_array_almost_equal(W, elm.nnet.get_neurons()[0][2])

    def test_AddNeurons_InitDefault_BiasWNotZero(self):
        elm = ELM(2, 1)
        elm.add_neurons(3, "sigm")
        W = elm.nnet.get_neurons()[0][2]
        bias = elm.nnet.get_neurons()[0][3]
        self.assertGreater(np.sum(np.abs(W)), 0.001)
        self.assertGreater(np.sum(np.abs(bias)), 0.001)

    def test_AddNeurons_InitTwiceBiasW_CorrectlyMerged(self):
        elm = ELM(2, 1)
        W1 = np.random.rand(2, 3)
        W2 = np.random.rand(2, 4)
        bias1 = np.random.rand(3, )
        bias2 = np.random.rand(4, )
        elm.add_neurons(3, "sigm", W1, bias1)
        elm.add_neurons(4, "sigm", W2, bias2)
        np.testing.assert_array_almost_equal(np.hstack((W1, W2)), elm.nnet.get_neurons()[0][2])
        np.testing.assert_array_almost_equal(np.hstack((bias1, bias2)), elm.nnet.get_neurons()[0][3])

    def test_Str_Works(self):
        elm = ELM(1, 1)
        s = "%s" % elm
        self.assertIn("ELM with 1 inputs and 1 output", s)

    def test_StrCustomNeurons_DisplaysName(self):
        elm = ELM(1, 1)
        func = np.sin
        elm.add_neurons(1, func)
        s_elm = "%s" % elm
        self.assertIn("sin", s_elm)

    def test_ELMWithBatch_SetsBatch(self):
        elm = ELM(1, 1, batch=123)
        self.assertEqual(123, elm.batch)

    def test_TrainWithBatch_OverwritesBatch(self):
        elm = ELM(1, 1, batch=123)
        X = np.array([1, 2, 3])
        T = np.array([1, 2, 3])
        elm.add_neurons(1, "lin")
        elm.train(X, T, batch=234)
        self.assertEqual(234, elm.batch)

    def test_Classification_Works(self):
        elm = ELM(1, 2)
        X = np.array([1, 2, 3, 4, 5, 6])
        T = np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]])
        elm.add_neurons(1, "lin")
        elm.train(X, T, 'c')

    def test_Classification_WorksCorreclty(self):
        elm = ELM(1, 2)
        X = np.array([-1, -0.6, -0.3, 0.3, 0.6, 1])
        T = np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]])
        elm.add_neurons(1, "lin")
        elm.train(X, T, 'c')
        Y = elm.predict(X)
        print Y
        self.assertGreater(Y[0, 0], Y[0, 1])
        self.assertLess(Y[5, 0], Y[5, 1])

    def test_MultiLabelClassification_Works(self):
        elm = ELM(1, 2)
        X = np.array([1, 2, 3, 4, 5, 6])
        T = np.array([[1, 1], [1, 0], [1, 0], [0, 1], [0, 1], [1, 1]])
        elm.add_neurons(1, "lin")
        elm.train(X, T, 'ml')
        elm.train(X, T, 'mc')

    def test_WeightedClassification_Works(self):
        elm = ELM(1, 2)
        X = np.array([1, 2, 3, 1, 2, 3])
        T = np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]])
        elm.add_neurons(1, "lin")
        elm.train(X, T, 'wc', w=(1, 1))

    def test_WeightedClassification_DefaultWeightsWork(self):
        elm = ELM(1, 2)
        X = np.array([1, 2, 3, 1, 2, 3])
        T = np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]])
        elm.add_neurons(1, "lin")
        elm.train(X, T, 'wc')

    def test_WeightedClassification_ClassWithLargerWeightWins(self):
        elm = ELM(1, 2)
        X = np.array([1, 2, 3, 1, 2, 3])
        T = np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]])
        elm.add_neurons(1, "lin")
        elm.train(X, T, 'wc', w=(1, 0.1))
        Y = elm.predict(X)
        self.assertGreater(Y[0, 0], Y[0, 1])
        self.assertGreater(Y[1, 0], Y[1, 1])
        self.assertGreater(Y[2, 0], Y[2, 1])

    def test_RegressionError_Works(self):
        T = np.array([1, 2, 3])
        Y = np.array([1.1, 2.2, 3.3])
        err1 = np.mean((T - Y) ** 2)
        elm = ELM(1, 1)
        e = elm.error(T, Y)
        np.testing.assert_allclose(e, err1)

    def test_ClassificationError_Works(self):
        X = np.array([1, 2, 3])
        T = np.array([[0, 1], [0, 1], [1, 0]])
        Y = np.array([[0, 1], [0.4, 0.6], [0, 1]])
        elm = ELM(1, 2, classification='c')
        elm.add_neurons(1, "lin")
        e = elm.error(T, Y)
        np.testing.assert_allclose(e, 1.0 / 3)

    def test_WeightedClassError_Works(self):
        T = np.array([[0, 1], [0, 1], [1, 0]])
        Y = np.array([[0, 1], [0.4, 0.6], [0, 1]])
        # here class 0 is totally incorrect, and class 1 is totally correct
        w = (9, 1)
        elm = ELM(1, 2, classification="wc", w=w)
        elm.add_neurons(1, "lin")
        e = elm.error(T, Y)
        np.testing.assert_allclose(e, 0.9)

    def test_MultiLabelClassError_Works(self):
        X = np.array([1, 2, 3])
        T = np.array([[0, 1], [1, 1], [1, 0]])
        Y = np.array([[0.4, 0.6], [0.8, 0.6], [1, 1]])
        elm = ELM(1, 2, classification="ml")
        elm.add_neurons(1, "lin")
        e = elm.error(T, Y)
        np.testing.assert_allclose(e, 1.0 / 6)

    def test_ProjectELM_WorksCorrectly(self):
        X = np.array([[1], [2], [3]])
        elm = ELM(1, 1)
        elm.add_neurons(1, "tanh", np.array([[1]]), np.array([0]))
        H = elm.project(X)
        np.testing.assert_allclose(H, np.tanh(X))

    def test_InitELM_SetNorm(self):
        nr = 0.03
        elm = ELM(1, 1, norm=nr)
        self.assertEqual(nr, elm.nnet.norm)

    def test_PrecisionELM_UsesPrecision(self):
        elm1 = ELM(1, 1, precision='32')
        self.assertIs(elm1.nnet.precision, np.float32)
        elm2 = ELM(1, 1, precision='single')
        self.assertIs(elm2.nnet.precision, np.float32)
        elm3 = ELM(1, 1, precision=np.float32)
        self.assertIs(elm3.nnet.precision, np.float32)
        elm4 = ELM(1, 1, precision='64')
        self.assertIs(elm4.nnet.precision, np.float64)
        elm5 = ELM(1, 1, precision='double')
        self.assertIs(elm5.nnet.precision, np.float64)
        elm6 = ELM(1, 1, precision=np.float64)
        self.assertIs(elm6.nnet.precision, np.float64)
        elm7 = ELM(1, 1)  # default double precision
        self.assertIs(elm7.nnet.precision, np.float64)
        elm8 = ELM(1, 1, precision="lol")  # default double precision
        self.assertIs(elm8.nnet.precision, np.float64)

    def test_ELM_SaveLoad(self):
        X = np.array([1, 2, 3, 1, 2, 3])
        T = np.array([[1, 0], [1, 0], [1, 0], [0, 1], [0, 1], [0, 1]])
        elm = ELM(1, 2, precision='32', norm=0.02)
        elm.add_neurons(1, "lin")
        elm.add_neurons(2, "tanh")
        elm.train(X, T, "wc", w=(0.7, 0.3))
        B1 = elm.nnet.get_B()
        try:
            f, fname = tempfile.mkstemp()
            elm.save(fname)
            elm2 = ELM(3, 3)
            elm2.load(fname)
        finally:
            os.close(f)
        self.assertEqual(elm2.nnet.inputs, 1)
        self.assertEqual(elm2.nnet.outputs, 2)
        self.assertEqual(elm2.classification, "wc")
        self.assertIs(elm.precision, np.float32)
        self.assertIs(elm2.precision, np.float64)  # precision has changed
        np.testing.assert_allclose(np.array([0.7, 0.3]), elm2.wc)
        np.testing.assert_allclose(0.02, elm2.nnet.norm)
        np.testing.assert_allclose(B1, elm2.nnet.get_B())
        self.assertEqual(elm2.nnet.get_neurons()[0][1], "lin")
        self.assertEqual(elm2.nnet.get_neurons()[1][1], "tanh")

    def test_SaveELM_WrongFile(self):
        elm = ELM(1, 1)
        try:
            f, fname = tempfile.mkstemp()
            self.assertRaises(IOError, elm.save, os.path.dirname(fname) + "olo/lo")
        finally:
            os.close(f)

    def test_LoadELM_WrongFile(self):
        elm = ELM(1, 1)
        try:
            f, fname = tempfile.mkstemp()
            self.assertRaises(IOError, elm.load, fname + "ololo2")
        finally:
            os.close(f)

    def test_ConfusionELM_Classification(self):
        T = np.array([[1, 0], [1, 0], [0, 1], [0, 1]])
        Y = np.array([[0, 1], [0, 1], [1, 0], [0, 1]])
        elm = ELM(1, 2)
        elm.classification = "c"
        C = elm.confusion(T, Y)
        np.testing.assert_allclose(C, np.array([[0, 2], [1, 1]]))

    def test_ConfusionELM_Multilabel(self):
        T = np.array([[1, 0], [1, 0], [0, 1], [0, 1]])
        Y = np.array([[1, 1], [1, 0], [0, 1], [0, 1]])
        elm = ELM(1, 2)
        elm.classification = "ml"
        C = elm.confusion(T, Y)
        np.testing.assert_allclose(C, np.array([[2, 1], [0, 2]]))

    def test_MRSR_Works(self):
        X = np.random.rand(10, 3)
        T = np.random.rand(10, 2)
        elm = ELM(3, 2)
        elm.add_neurons(5, "tanh")
        elm.train(X, T, "LOO", "OP")

    def test_MRSR2_Works(self):
        X = np.random.rand(20, 9)
        T = np.random.rand(20, 12)
        elm = ELM(9, 12)
        elm.add_neurons(5, "tanh")
        elm.train(X, T, "LOO", "OP")








# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np
import cPickle
from tables import open_file
from nnets.slfn import SLFN
from nnets.slfn_python import SLFNPython
from hpelm.modules import mrsr, mrsr2
from mss_v import train_v
from mss_cv import train_cv
from mss_loo import train_loo


class ELM(object):
    """Interface for training Extreme Learning Machines (ELM).

    Args:
        inputs (int): dimensionality of input data, or number of data features
        outputs (int): dimensionality of output data, or number of classes
        classification ('c'/'wc'/'ml', optional): train ELM for classfication ('c') / weighted classification ('wc') /
            multi-label classification ('ml'). For weighted classification you can provide weights in `w`. ELM will
            compute and use the corresponding classification error instead of Mean Squared Error.
        w (vector, optional): weights vector for weighted classification, lenght (`outputs` * 1).
        batch (int, optional): batch size for data processing in ELM, reduces memory requirements. Does not work
            for model structure selection (validation, cross-validation, Leave-One-Out). Can be changed later directly
            as a class attribute.
        accelerator ("GPU"/"basic", optional): type of accelerated ELM to use: None, 'GPU', 'basic', ...
        precision (optional): data precision to use, supports signle ('single', '32' or numpy.float32) or double
            ('double', '64' or numpy.float64). Single precision is faster but may cause numerical errors. Majority
            of GPUs work in single precision. Default: **double**.
        norm (double, optinal): L2-normalization parameter, **None** gives the default value.
        tprint (int, optional): ELM reports its progess every `tprint` seconds or after every batch,
            whatever takes longer.

    Class attributes; attributes that simply store initialization or `train()` parameters are omitted.

    Attributes:
        nnet (object): Implementation of neural network with computational methods, but without
            complex logic. Different implementations are given by different classes: for Python, for GPU, etc.
            See ``hpelm.nnets`` folder for particular files. You can implement your own computational algorithm
            by inheriting from ``hpelm.nnets.SLFN`` and overwriting some methods.
        flist (list of strings): Awailable types of neurons, use them when adding new neurons.

    Note:
        Below the 'matrix' type means a 2-dimensional Numpy.ndarray.
    """
    # TODO: note about HDF5 instead of matrix for Matlab compatibility

    def __init__(self, inputs, outputs, classification="", w=None, batch=1000, accelerator=None,
                 precision='double', norm=None, tprint=5):
        assert isinstance(inputs, (int, long)), "Number of inputs must be integer"
        assert isinstance(outputs, (int, long)), "Number of outputs must be integer"
        assert batch > 0, "Batch should be positive"

        self.batch = int(batch)
        self.precision = np.float64

        if precision in (np.float32, np.float64):
            self.precision = precision
        elif 'double' in precision.lower() or '64' in precision:
            self.precision = np.float64
        elif 'single' in precision or '32' in precision:
            self.precision = np.float32
        else:
            print "Unknown precision parameter: %s, using double precision" % precision

        # create SLFN solver to do actual computations
        self.accelerator = accelerator
        if accelerator is "GPU":
            print "Using CUDA GPU acceleration with Scikit-CUDA"
            from nnets.slfn_skcuda import SLFNSkCUDA
            self.nnet = SLFNSkCUDA(inputs, outputs, precision=self.precision, norm=norm)
        elif accelerator is "basic":
            print "Using slower basic Python solver"
            self.nnet = SLFN(inputs, outputs, precision=self.precision, norm=norm)
        else: # double precision Numpy solver
            self.nnet = SLFNPython(inputs, outputs, precision=self.precision, norm=norm)

        # init other stuff
        self.classification = None  # train ELM for classification
        if classification.lower() in ("c", "wc", "ml", "mc"):  # allow 'mc'=='ml' for compatibility
            self.classification = classification.replace("mc", "ml")
        self.wc = None  # weighted classification weights
        if w is not None:
            w = np.array(w)
            assert len(w) == outputs, "Number of class weights must be equal to the number of classes"
            self.wc = w
        self.opened_hdf5 = []  # list of opened HDF5 files, they are closed in ELM descructor
        self.ranking = None
        self.kmax_op = None
        self.tprint = tprint  # time intervals in seconds to report ETA
        self.flist = ("lin", "sigm", "tanh", "rbf_l1", "rbf_l2", "rbf_linf")  # supported neuron types

    def __str__(self):
        s = "ELM with %d inputs and %d outputs\n" % (self.nnet.inputs, self.nnet.outputs)
        s += "Hidden layer neurons: "
        for n, func, _, _ in self.nnet.neurons:
            s += "%d %s, " % (n, func)
        s = s[:-2]
        return s

    def _train_parse_args(self, args, kwargs):
        """Parse training args and set corresponding class variables."""
        assert len(self.nnet.neurons) > 0, "Add neurons to ELM before training it"
        args = [a.upper() for a in args]  # make all arguments upper case

        # reset parameters
        self.nnet.reset()  # remove previous training
        self.ranking = None
        self.kmax_op = None
        self.classification = None  # c / wc / ml
        self.wc = None  # weigths for weighted classification

        # check exclusive parameters
        assert len(set(args).intersection({"V", "CV", "LOO"})) <= 1, "Use only one of V / CV / LOO"
        msg = "Use only one of: C (classification) / WC (weighted classification) / ML (multi-label classification)"
        assert len(set(args).intersection({"C", "WC", "ML", "MC"})) <= 1, msg

        # parse parameters
        for a in args:
            if a == "OP":
                self.ranking = "OP"
                if "kmax" in kwargs.keys():
                    self.kmax_op = int(kwargs["kmax"])
            if a == "C":
                assert self.nnet.outputs > 1, "Classification outputs must have 1 output per class"
                self.classification = "c"
            if a == "WC":
                assert self.nnet.outputs > 1, "Classification outputs must have 1 output per class"
                self.classification = "wc"
                if 'w' in kwargs.keys():
                    w = np.array(kwargs['w'])
                    assert len(w) == self.nnet.outputs, "Number of class weights must be equal to the number of classes"
                    self.wc = w
            if a == "ML" or a == "MC":
                assert self.nnet.outputs > 1, "Classification outputs must have 1 output per class"
                self.classification = "ml"
            if a == "R":
                self.classification = None  # reset to regression

        if "batch" in kwargs.keys():
            self.batch = int(kwargs["batch"])

    def train(self, X, T, *args, **kwargs):
        """Universal training interface for ELM model with model structure selection.

        Model structure selection takes more time and requires all data to fit into memory. Optimal pruning ('OP',
        effectively an L1-regularization) takes the most time but gives the smallest and best performing model.
        Choosing a classification forces ELM to use classification error in model structure selection,
        and in `error()` method output.

        Args:
            X (matrix): input data matrix, size (N * `inputs`)
            T (matrix): outputs data matrix, size (N * `outputs`)
            'V'/'CV'/'LOO' (sting, choose one): model structure selection: select optimal number of neurons using
                a validation set ('V'), cross-validation ('CV') or Leave-One-Out ('LOO')
            'OP' (string, use with 'V'/'CV'/'LOO'): choose best neurons instead of random ones, training takes longer;
                equivalent to L1-regularization
            'c'/'wc'/'ml'/'r' (string, choose one): train ELM for classification ('c'), classification with weighted
                classes ('wc'), multi-label classification ('ml') with several correct classes per data sample, or
                regression ('r') without any classification. In classification, number of `outputs` is the number
                of classes; correct class(es) for each sample has value 1 and incorrect classes have 0.
                Overwrites parameters given an ELM initialization time.

        Keyword Args:
            Xv (matrix, use with 'V'): validation set input data, size (Nv * `inputs`)
            Tv (matrix, use with 'V'): validation set outputs data, size (Nv * `outputs`)
            k (int, use with 'CV'): number of splits for cross-validation, k>=3
            kmax (int, optional, use with 'OP'): maximum number of neurons to keep in ELM
            batch (int, optional): batch size for ELM, overwrites batch size from the initialization
        """
        X, T = self._checkdata(X, T)
        self._train_parse_args(args, kwargs)

        # TODO: test upper case and lower case 'V', ...
        # train ELM with desired model structure selection
        if "V" in args:  # use validation set
            assert "Xv" in kwargs.keys(), "Provide validation dataset (Xv)"
            assert "Tv" in kwargs.keys(), "Provide validation outputs (Tv)"
            Xv = kwargs['Xv']
            Tv = kwargs['Tv']
            Xv, Tv = self._checkdata(Xv, Tv)
            train_v(self, X, T, Xv, Tv)
        elif "CV" in args:  # use cross-validation
            assert "k" in kwargs.keys(), "Provide Cross-Validation number of splits (k)"
            k = kwargs['k']
            assert k >= 3, "Use at least k=3 splits for Cross-Validation"
            train_cv(self, X, T, k)
        elif "LOO" in args:  # use Leave-One-Out error on training set
            train_loo(self, X, T)
        else:  # basic training algorithm
            self.add_data(X, T)
            self.nnet.solve()
        # TODO: Adaptive ELM model for timeseries (someday)

    def add_data(self, X, T):
        """Feed new training data (X,T) to ELM model in batches; does not solve ELM itself.

        Helper method that updates intermediate solution parameters HH and HT, which are used for solving ELM later.
        Updates accumulate, so this method can be called multiple times with different parts of training data.
        To reset accumulated training data, use `ELM.nnet.reset()`.

        For training an ELM use `ELM.train()` instead.

        Args:
            X (matrix): input training data
            T (matrix): output training data
        """
        # initialize batch size
        nb = int(np.ceil(float(X.shape[0]) / self.batch))
        wc_vector = None

        # find automatic weights if none are given
        if self.classification == "wc" and self.wc is None:
            ns = T.sum(axis=0).astype(self.precision)  # number of samples in classes
            self.wc = ns.sum() / ns  # weights of classes

        for X0, T0 in zip(np.array_split(X, nb, axis=0),
                          np.array_split(T, nb, axis=0)):
            if self.classification == "wc":
                wc_vector = self.wc[np.where(T0 == 1)[1]]  # weights for samples in the batch
            self.nnet.add_batch(X0, T0, wc_vector)

    def add_neurons(self, number, func, W=None, B=None):
        """Adds neurons to ELM model. ELM is created empty, and needs some neurons to work.

        Add neurons to an empty ELM model, or add more neurons to a model that already has some.

        Random weights `W` and biases `B` are generated automatically if not provided explicitly.
        Maximum number of neurons is limited by the available RAM and computational power, a sensible limit
        would be 1000 neurons for an average size dataset and 15000 for the largest datasets. ELM becomes slower after
        3000 neurons because computational complexity is proportional to a qube of number of neurons.

        This method checks and prepares neurons, they are actually stored in `solver` object.

        Args:
            number (int): number of neurons to add
            func (string): type of neurons: "lin" for linear, "sigm" or "tanh" for non-linear,
                "rbf_l1", "rbf_l2" or "rbf_linf" for radial basis function neurons.
            W (matrix, optional): random projection matrix size (`inputs` * `number`). For 'rbf_' neurons,
                W stores centroids of radial basis functions in transposed form.
            B (vector, optional): bias vector of size (`number` * 1), a 1-dimensional Numpy.ndarray object.
                For 'rbf_' neurons, B gives widths of radial basis functions.
        """
        assert isinstance(number, int), "Number of neurons must be integer"
        assert (func in self.flist or isinstance(func, np.ufunc)),\
            "'%s' neurons not suppored: use a standard neuron function or a custom <numpy.ufunc>" % func
        assert isinstance(W, (np.ndarray, type(None))), "Projection matrix (W) must be a Numpy ndarray"
        assert isinstance(B, (np.ndarray, type(None))), "Bias vector (B) must be a Numpy ndarray"
        inputs = self.nnet.inputs

        # default neuron initializer
        if W is None:
            if func == "lin":  # copying input features for linear neurons
                number = min(number, inputs)  # cannot have more linear neurons than features
                W = np.eye(inputs, number)
            else:
                W = np.random.randn(inputs, number)
                if func not in ("rbf_l1", "rbf_l2", "rbf_linf"):
                    W *= 3.0 / inputs**0.5  # high dimensionality fix
        if B is None:
            B = np.random.randn(number)
            if func in ("rbf_l2", "rbf_l1", "rbf_linf"):
                B = np.abs(B)
                B *= inputs
            if func == "lin":
                B = np.zeros((number,))
        msg = "W must be size [inputs, neurons] (expected [%d,%d])" % (inputs, number)
        assert W.shape == (inputs, number), msg
        assert B.shape == (number,), "B must be size [neurons] (expected [%d])" % number
        # set to correct precision
        W = W.astype(self.precision)
        B = B.astype(self.precision)

        # add prepared neurons to the model
        self.nnet.add_neurons(number, func, W, B)

    def error(self, T, Y):
        """Calculate error of model predictions.

        Computes Mean Squared Error (MSE) between model predictions Y and true outputs T.
        For classification, computes mis-classification error.
        For multi-label classification, correct classes are all with Y>0.5.

        For weighted classification the error is an average weighted True Positive Rate,
        or percentage of correctly predicted samples for each class, multiplied by weight
        of that class and averaged. If you want something else, just write it yourself :)
        See https://en.wikipedia.org/wiki/Confusion_matrix for details.

        Another option is to use scikit-learn's performance metrics. Transform `Y` and `T` into scikit's
        format by ``y_true = T.argmax[1]``, ``y_pred = Y.argmax[1]``.
        http://scikit-learn.org/stable/modules/classes.html#module-sklearn.metrics

        Args:
            T (matrix): true outputs.
            Y (matrix): ELM model predictions, can be computed with `predict()` function.

        Returns:
            e (double): MSE for regression / classification error for classification.
        """
        _, T = self._checkdata(None, T)
        _, Y = self._checkdata(None, Y)
        return self._error(T, Y)

    def confusion(self, T, Y):
        """Computes confusion matrix for classification.

        Confusion matrix :math:`C` such that element :math:`C_{i,j}` equals to the number of observations known
        to be class :math:`i` but predicted to be class :math:`j`.

        Args:
            T (matrix): true outputs or classes, size (N * `outputs`)
            Y (matrix): predicted outputs by ELM model, size (N * `outputs`)

        Returns:
            conf (matrix): confusion matrix, size (`outputs` * `outputs`)
        """
        # TODO: ELM type can be assigned at creation time: "c", "wc", "ml"
        assert self.classification in ("c", "wc", "ml"), "Confusion matrix works only for regression"
        _, T = self._checkdata(None, T)
        _, Y = self._checkdata(None, Y)
        N = T.shape[0]
        nb = int(np.ceil(float(N) / self.batch))  # number of batches

        C = self.nnet.outputs
        conf = np.zeros((C, C))

        if self.classification in ("c", "wc"):
            for b in xrange(nb):
                start = b*self.batch
                stop = min((b+1)*self.batch, N)
                Tb = np.array(T[start:stop]).argmax(1)
                Yb = np.array(Y[start:stop]).argmax(1)
                for c1 in xrange(C):
                    for c1h in xrange(C):
                        conf[c1, c1h] += np.logical_and(Tb == c1, Yb == c1h).sum()
        elif self.classification == "ml":
            for b in xrange(nb):
                start = b*self.batch
                stop = min((b+1)*self.batch, N)
                Tb = np.array(T[start:stop]) > 0.5
                Yb = np.array(Y[start:stop]) > 0.5
                for c1 in xrange(C):
                    for c1h in xrange(C):
                        conf[c1, c1h] += np.sum(Tb[:, c1] * Yb[:, c1h])
        return conf

    def project(self, X):
        """Get ELM's hidden layer representation of input data.

        Args:
            X (matrix): input data, size (N * `inputs`)

        Returns:
            H (matrix): hidden layer representation matrix, size (N * number_of_neurons)
        """
        X, _ = self._checkdata(X, None)
        H = self.nnet._project(X)
        return H

    def predict(self, X):
        """Predict outputs Y for the given input data X.

        Args:
            X (matrix): input data of size (N * `inputs`)

        Returns:
            Y (matrix): output data or predicted classes, size (N * `outputs`).
        """
        X, _ = self._checkdata(X, None)
        Y = self.nnet._predict(X)
        return Y

    def save(self, fname):
        """Save ELM model with current parameters.

        Model does not save a particular solver, precision batch size. They are obtained from
        a new ELM when loading the model (so one can switch to another solver, for instance).

        Also ranking and max number of neurons are not saved, because they
        are runtime training info irrelevant after the training completes.

        Args:
            fname (string): filename to save model into.
        """
        assert isinstance(fname, basestring), "Model file name must be a string"
        m = {"inputs": self.nnet.inputs,
             "outputs": self.nnet.outputs,
             "Classification": self.classification,
             "Weights_WC": self.wc,
             "neurons": self.nnet.neurons,
             "norm": self.nnet.norm,  # W and bias are here
             "Beta": self.nnet.get_B()}
        try:
            cPickle.dump(m, open(fname, "wb"), -1)
        except IOError:
            raise IOError("Cannot create a model file at: %s" % fname)

    def load(self, fname):
        """Load ELM model data from a file.

        Load requires an ``ELM`` object, and it uses solver type, precision and batch size from that ELM object.

        Args:
            fname (string): filename to load model from.
        """
        assert isinstance(fname, basestring), "Model file name must be a string"
        try:
            m = cPickle.load(open(fname, "rb"))
        except IOError:
            raise IOError("Model file not found: %s" % fname)
        inputs = m["inputs"]
        outputs = m["outputs"]
        self.classification = m["Classification"]
        self.wc = m["Weights_WC"]

        # create a new solver and load neurons / Beta into it with correct precision
        if self.accelerator is None:
            self.nnet = SLFN(inputs, outputs, precision=self.precision)
        for number, func, W, B in m["neurons"]:
            self.nnet.add_neurons(number, func, W.astype(self.precision), B.astype(self.precision))
        self.nnet.norm = m["norm"]
        self.nnet.set_B(np.array(m["Beta"], dtype=self.precision))

    def __del__(self):
        # Closes any HDF5 files opened during HPELM usage.
        for h5 in self.opened_hdf5:
            h5.close()

    def _error(self, T, Y, R=None):
        """Returns regression/classification/multiclass error, also for PRESS.

        An ELM-specific error with PRESS support.
        """
        if R is None:  # normal classification error
            if self.classification == "c":
                err = np.not_equal(Y.argmax(1), T.argmax(1)).mean()
            elif self.classification == "wc":  # weighted classification
                c = T.shape[1]
                errc = np.zeros(c)
                for i in xrange(c):  # per-class MSE
                    idx = np.where(T[:, i] == 1)[0]
                    if len(idx) > 0:
                        errc[i] = np.not_equal(Y[idx].argmax(1), i).mean()
                err = np.sum(errc * self.wc) / np.sum(self.wc)
            elif self.classification == "ml":
                err = np.not_equal(Y > 0.5, T > 0.5).mean()
            else:
                err = np.mean((Y - T)**2)
        else:  # LOO_PRESS error
            if self.classification == "c":
                err = np.not_equal(Y.argmax(1), T.argmax(1)) / R.ravel()
                err = np.mean(err**2)
            elif self.classification == "wc":  # balanced classification
                c = T.shape[1]
                errc = np.zeros(c)
                for i in xrange(c):  # per-class MSE
                    idx = np.where(T[:, i] == 1)[0]
                    if len(idx) > 0:
                        t = np.not_equal(Y[idx].argmax(1), i) / R[idx].ravel()
                        errc[i] = np.mean(t**2)
                err = np.mean(errc * self.wc)
            elif self.classification == "ml":
                err = np.not_equal(Y > 0.5, T > 0.5) / R.reshape((-1, 1))
                err = np.mean(err**2)
            else:
                err = (Y - T) / R.reshape((-1, 1))
                err = np.mean(err**2)
        assert not np.isnan(err), "Error is NaN at %s" % self.classification
        return np.float64(err)

    def _ranking(self, L, H=None, T=None):
        """Return ranking of hidden neurons; random or OP.

        Args:
            L (int): number of neurons
            H (matrix): hidden layer representation matrix needed for optimal pruning
            T (matrix): outputs matrix needed for optimal pruning

        Returns:
            rank (vector): ranking of neurons
            L (int): number of selected neurons, can be changed by `self.kmax_op`
        """
        if self.ranking == "OP":  # optimal ranking (L1 normalization)
            assert H is not None and T is not None, "Need H and T to perform optimal pruning"
            if self.kmax_op is not None:  # apply maximum number of neurons
                L = min(self.kmax_op, L)
            if T.shape[1] < 10:  # fast mrsr for less outputs but O(2^t) in outputs
                rank = mrsr(H, T, L)
            else:  # slow mrsr for many outputs but O(t) in outputs
                rank = mrsr2(H, T, L)
        else:  # random ranking
            rank = np.arange(L)
            np.random.shuffle(rank)
        return rank, L

    def _checkdata(self, X, T):
        """Checks data variables and fixes matrix dimensionality issues.
        """
        if X is not None:
            if isinstance(X, basestring):  # open HDF5 file
                try:
                    h5 = open_file(X, "r")
                except:
                    raise IOError("Cannot read HDF5 file at %s" % X)
                self.opened_hdf5.append(h5)
                node = None
                for node in h5.walk_nodes():
                    pass  # find a node with whatever name
                if node:
                    X = node
                else:
                    raise IOError("Empty HDF5 file at %s" % X)
            else:
                # assert isinstance(X, np.ndarray) and
                assert X.dtype.kind not in "OSU", "X must be a numerical numpy array"
                if len(X.shape) == 1:
                    X = X.reshape(-1, 1)
            assert len(X.shape) == 2, "X must have 2 dimensions"
            assert X.shape[1] == self.nnet.inputs, "X has wrong dimensionality: expected %d, found %d" % \
                                                   (self.nnet.inputs, X.shape[1])

        if T is not None:
            if isinstance(T, basestring):  # open HDF5 file
                try:
                    h5 = open_file(T, "r")
                except IOError:
                    raise IOError("Cannot read HDF5 file at %s" % T)
                self.opened_hdf5.append(h5)
                node = None
                for node in h5.walk_nodes():
                    pass  # find a node with whatever name
                if node:
                    T = node
                else:
                    raise IOError("Empty HDF5 file at %s" % X)
            else:
                # assert isinstance(T, np.ndarray) and
                assert T.dtype.kind not in "OSU", "T must be a numerical numpy array"
                if len(T.shape) == 1:
                    T = T.reshape(-1, 1)
            assert len(T.shape) == 2, "T must have 2 dimensions"

            assert T.shape[1] == self.nnet.outputs, "T has wrong dimensionality: expected %d, found %d" % \
                                                    (self.nnet.outputs, T.shape[1])

        if (X is not None) and (T is not None):
            assert X.shape[0] == T.shape[0], "X and T cannot have different number of samples"

        return X, T



# -*- coding: utf-8 -*-
"""This is a fast Python implementation of SLFN.

Created on Sun Sep  6 11:18:55 2015
@author: akusok
"""

import os
import platform

from slfn import SLFN
import numpy as np
from scipy.linalg import blas, lapack


class SLFNPython(SLFN):
    """Single Layer Feed-forward Network (SLFN), the neural network that ELM trains.
    """

    def __init__(self, inputs, outputs, norm=None, precision=np.float64):

        super(SLFNPython, self).__init__(inputs, outputs, norm, precision)
        # get correct BLAS/LAPACK functions for precision
        if precision is np.float32:
            self.syrk = blas.ssyrk
            self.posv = lapack.sposv
        elif precision is np.float64:
            self.syrk = blas.dsyrk
            self.posv = lapack.dposv


    def add_batch(self, X, T, wc=None):
        """Add a batch using Symmetric Rank-K matrix update for HH.
        """
        H = self._project(X)
        T = T.astype(self.precision)
        if wc is not None:  # apply weights if given
            w = np.array(wc**0.5, dtype=self.precision)[:, None]  # re-shape to column matrix
            H *= w
            T *= w

        if self.HH is None:  # initialize space for self.HH, self.HT
            self.HH = np.zeros((self.L, self.L), dtype=self.precision)
            self.HT = np.zeros((self.L, self.outputs), dtype=self.precision)
            np.fill_diagonal(self.HH, self.norm)

        #self.syrk(1, H.T, 1, self.HH, trans=0, overwrite_c=1)  # 'overwrite_c' does not work
        self.HH = self.syrk(1, H.T, 1, self.HH, trans=0)  # self.HH += np.dot(H.T, H)
        self.HT += np.dot(H.T, T)

    def solve_corr(self, HH, HT):
        """Compute output weights B for given HH and HT.

        Simple but inefficient version, see a better one in solver_python.

        Args:
            HH (matrix): covariance matrix of hidden layer represenation H, size (`L` * `L`)
            HT (matrix): correlation matrix between H and outputs T, size (`L` * `outputs`)
        """
        _, B, info = self.posv(HH, HT)
        if info > 0:
            print "Covariance matrix is not full rank; solving with SVD (slow)"
            print "This happened because you have duplicated or too many neurons"
            HH = HH + np.triu(HH, k=1).T
            B = np.linalg.lstsq(HH, HT)[0]
        return B


    def get_corr(self):
        """Return current correlation matrices.
        """
        HH = self.HH + np.triu(self.HH, k=1).T
        return HH, self.HT



# -*- coding: utf-8 -*-
"""Basic SLFN solver that follows paper notations, defines interface for all solvers.


Created on Sun Sep  6 11:18:55 2015
@author: akusok
"""

import os
import platform

import numpy as np
from scipy.spatial.distance import cdist


class SLFN(object):
    """Single Layer Feed-forward Network (SLFN), the neural network that ELM trains.

    This implementation is not the fastest but very simple, and it defines interface.
    Gives correct output, other solvers should provide the same output as this guy.

    Args:
        outputs (int): number of outputs, or classes for classification
        norm (double): output weights normalization parameter (Tikhonov normalizaion, or
            ridge regression), large values provides smaller (= better) weights but worse model accuracy
        precision (Numpy.float32/64): solver precision, float32 is faster but may be worse, most GPU
            work fast only in float32.

    Attributes:
        neurons (list): a list of different types of neurons, initially empty. One neuron type is a tuple
            ('number of neurons', 'function_type', W, Bias), `neurons` is a list of [neuron_type_1, neuron_type_2, ...].
        func (dict): a dictionary of transformation function type, key is a neuron type (= function name)
            and value is the function itself. A single function takes input parameters X, W, B, and outputs
            corresponding H for its neuron type.
        HH, HT (matrix): intermediate covariance matrices used in ELM solution. Can be computed and stored in GPU
            memory for accelerated SLFN. They are not needed once ELM is solved and they can take a lot of memory
            with large numbers of neurons, so one can delete them with `reset()` method. They are omitted when
            an ELM model is saved.
        B (matrix): output solution matrix of SLFN. A trained ELM needs only `neurons` and `B` to predict outputs
            for new input data.
    """

    def __init__(self, inputs, outputs, norm=None, precision=np.float64):
        """Initialize class variables and transformation functions.
        """
        self.inputs = inputs  # not used here, but logically are a part of SLFN
        self.outputs = outputs  # number of outputs, also number of classes (thus 'c')
        self.precision = precision
        if norm is None:
            norm = 50*np.finfo(precision).eps  # 50 times machine epsilon
        self.norm = norm
        # cannot use a dictionary for neurons, because its iteration order is not defined
        self.neurons = []  # list of all neurons in normal Numpy form
        self.L = None  # number of neurons
        self.B = None
        self.HH = None
        self.HT = None

        # transformation functions in HPELM, accessible by name
        self.func = {}
        self.func["lin"] = lambda X, W, B: np.dot(X, W) + B
        self.func["sigm"] = lambda X, W, B: 1 / (1 + np.exp(np.dot(X, W) + B))
        self.func["tanh"] = lambda X, W, B: np.tanh(np.dot(X, W) + B)
        self.func["rbf_l1"] = lambda X, W, B: np.exp(-cdist(X, W.T, "cityblock")**2 / B)
        self.func["rbf_l2"] = lambda X, W, B: np.exp(-cdist(X, W.T, "euclidean")**2 / B)
        self.func["rbf_linf"] = lambda X, W, B: np.exp(-cdist(X, W.T, "chebyshev")**2 / B)

    def add_neurons(self, number, func, W, B):
        """Add prepared neurons to the SLFN, merge with existing ones.

        Adds a number of specific neurons to SLFN network. Weights and biases
        must be provided for that function.

        If neurons of such type already exist, they are merged together.

        Args:
            number (int): the number of new neurons to add
            func (str): transformation function of hidden layer. Linear function creates a linear model.
            W (matrix): a 2-D matrix of neuron weights, size (`inputs` * `number`)
            B (vector): a 1-D vector of neuron biases, size (`number` * 1)
        """
        ntypes = [nr[1] for nr in self.neurons]  # existing types of neurons
        if func in ntypes:
            # add to an existing neuron type
            i = ntypes.index(func)
            nn0, _, W0, B0 = self.neurons[i]
            number = nn0 + number
            W = np.hstack((W0, W))
            B = np.hstack((B0, B))
            self.neurons[i] = (number, func, W, B)
        else:
            # create a new neuron type
            self.neurons.append((number, func, W, B))
        self.reset()
        self.B = None

    def reset(self):
        """ Resets intermediate training results, releases memory that they use.

        Keeps solution of ELM, so a trained ELM remains operational.
        Can be called to free memory after an ELM is trained.
        """
        self.L = sum([n[0] for n in self.neurons])  # get number of neurons
        self.HH = None
        self.HT = None

    def _project(self, X):
        """Projects X to H, an auxiliary function that implements a particular projection.

        For actual projection, use `ELM.project()` instead.

        Args:
            X (matrix): an input data matrix, size (N * `inputs`)

        Returns:
            H (matrix): an SLFN hidden layer representation, size (N * `L`) where 'L' is number of neurons
        """
        assert self.neurons is not None, "ELM has no neurons"
        X = X.astype(self.precision)
        return np.hstack([self.func[ftype](X, W, B) for _, ftype, W, B in self.neurons])

    def _predict(self, X):
        """Predict a batch of data. Auxiliary function that implements a particular prediction.

        For prediction, use `ELM.predict()` instead.

        Args:
            X (matrix): input data size (N * `inputs`)

        Returns:
            Y (matrix): predicted outputs size (N * `outputs`), always in float/double format.
        """
        assert self.B is not None, "Solve the task before predicting"
        H = self._project(X)
        Y = np.dot(H, self.B)
        return Y

    def add_batch(self, X, T, wc=None):
        """Add a batch of training data to an iterative solution, weighted if neeed.

        The batch is processed as a whole, the training data is splitted in `ELM.add_data()` method.
        With parameters HH_out, HT_out, the output will be put into these matrices instead of model.

        Args:
            X (matrix): input data matrix size (N * `inputs`)
            T (matrix): output data matrix size (N * `outputs`)
            wc (vector): vector of weights for data samples, one weight per sample, size (N * 1)
            HH_out, HT_out (matrix, optional): output matrices to add batch result into, always given together
        """
        H = self._project(X)
        T = T.astype(self.precision)
        if wc is not None:  # apply weights if given
            w = np.array(wc**0.5, dtype=self.precision)[:, None]  # re-shape to column matrix
            H *= w
            T *= w

        if self.HH is None:  # initialize space for self.HH, self.HT
            self.HH = np.zeros((self.L, self.L), dtype=self.precision)
            self.HT = np.zeros((self.L, self.outputs), dtype=self.precision)
            np.fill_diagonal(self.HH, self.norm)

        self.HH += np.dot(H.T, H)
        self.HT += np.dot(H.T, T)

    def solve(self):
        """Redirects to solve_corr, to avoid duplication of code.
        """
        self.B = self.solve_corr(self.HH, self.HT)

    def solve_corr(self, HH, HT):
        """Compute output weights B for given HH and HT.

        Simple but inefficient version, see a better one in solver_python.

        Args:
            HH (matrix): covariance matrix of hidden layer represenation H, size (`L` * `L`)
            HT (matrix): correlation matrix between H and outputs T, size (`L` * `outputs`)
        """
        HH_pinv = np.linalg.pinv(HH)
        B = np.dot(HH_pinv, HT)
        return B

    def _prune(self, idx):
        """Leave only neurons with the given indexes.
        """
        idx = list(idx)
        neurons = []
        for nold in self.neurons:
            k = nold[0]  # number of neurons
            ix1 = [i for i in idx if i < k]  # index for current neuron type
            idx = [i-k for i in idx if i >= k]
            func = nold[1]
            number = len(ix1)
            W = nold[2][:, ix1]
            bias = nold[3][ix1]
            neurons.append((number, func, W, bias))
        self.neurons = neurons
        # reset invalid parameters
        self.reset()
        self.B = None

    def get_B(self):
        """Return B as a numpy array.
        """
        return self.B

    def set_B(self, B):
        """Set B as a numpy array.

        Args:
            B (matrix): output layer weights matrix, size (`L` * `outputs`)
        """
        assert B.shape[0] == self.L, "Incorrect first dimension: %d expected, %d found" % (self.L, B.shape[0])
        assert B.shape[1] == self.outputs, "Incorrect output dimension: %d expected, %d found" % (self.outputs, B.shape[1])
        self.B = B.astype(self.precision)

    def get_corr(self):
        """Return current correlation matrices.
        """
        return self.HH, self.HT

    def set_corr(self, HH, HT):
        """Set pre-computed correlation matrices.

        Args:
            HH (matrix): covariance matrix of hidden layer represenation H, size (`L` * `L`)
            HT (matrix): correlation matrix between H and outputs T, size (`L` * `outputs`)
        """
        assert self.neurons is not None, "Add or load neurons before using ELM"
        assert HH.shape[0] == HH.shape[1], "HH must be a square matrix"
        msg = "Wrong HH dimension: (%d, %d) expected, %s found" % (self.L, self.L, HH.shape)
        assert HH.shape[0] == self.L, msg
        assert HH.shape[0] == HT.shape[0], "HH and HT must have the same number of rows (%d)" % self.L
        assert HT.shape[1] == self.outputs, "Number of columns in HT must equal number of outputs (%d)" % self.outputs
        self.HH = HH.astype(self.precision)
        self.HT = HT.astype(self.precision)

    def get_neurons(self):
        """Return current neurons.

        Returns:
            neurons (list of tuples (number/int, func/string, W/matrix, B/vector)): current neurons in the model
        """
        return self.neurons

    def fix_affinity(self):
        """Numpy processor core affinity fix.

        Fixes a problem if all Numpy processes are pushed to CPU core 0.
        """
        if "Linux" in platform.system():
            a = np.random.rand(3, 1)
            np.dot(a.T, a)
            pid = os.getpid()
            os.system("taskset -p 0xffffffff %d >/dev/null" % pid)



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 16 12:10:02 2015

@author: akusok
"""

from tables import open_file
import numpy as np
from scipy.spatial.distance import cdist

# TODO: use these parameters, or remove them

def rbf_param(data, k, kind="sqeuclidean"):
    """Calculates parameters for RBF neurons.

    :param data: - a matrix or an HDF5 file
    """
    if "l1" in kind:
        kind = "cityblock"
    elif "inf" in kind:
        kind = "chebyshev"
    else:
        kind = "sqeuclidean"
    if isinstance(data, basestring):
        h5 = open_file(data, "r")
        X = h5.root.data
    else:
        X = np.array(data)
        assert len(X.shape) == 2, "Data must be a 2-dim matrix"
    N = X.shape[0]
    Nk = min(10*k, N-1)
    dist = np.zeros((Nk,))

    ix = np.random.choice(N, size=Nk)
    for i in xrange(Nk):
        j = ix[i]
        if i == j:
            j += 1
        dist[i] = cdist(X[i][None, :], X[j][None, :], kind)
    m = dist.mean()
    s = dist.std()

    # fill centroids as random points
    ix = np.random.choice(N, size=k)
    W = np.empty((X.shape[1], k))
    for i in range(len(ix)):
        W[:, i] = X[ix[i]]

    # fill bias
    B = np.zeros((k,))
    i = 0
    while True:
        b0 = (np.random.rand()-0.5)*2*s + m
        if b0 > 0:
            B[i] = b0
            i += 1
            if i == k:
                break
    if isinstance(data, basestring):
        h5.close()

    return W, B



# -*- coding: utf-8 -*-
"""
Created on Mon Oct 27 17:48:33 2014

@author: akusok
"""

import numpy as np


def train_loo(self, X, T):
    """Model structure selection with Leave-One-Out (LOO) validation.

    Trains ELM, validates model with LOO and sets an optimal validated solution. Effect is similar to
    cross-validation with k==N, but ELM has explicit formula of solution for LOO without iterating k times.

    Args:
        self (ELM): ELM object that calls `train_v()`
        X (matrix): training set inputs
        T (matrix): training set outputs
    """

    H = self.nnet._project(X)
    self.add_data(X, T)
    HH, HT = self.nnet.get_corr()

    N = X.shape[0]
    L = self.nnet.L
    e = np.ones((L+1,)) * -1  # errors for all numbers of neurons
    rank, L = self._ranking(L, H, T)  # create ranking of neurons

    # PRESS_LOO
    P = np.linalg.inv(HH)
    Bt = np.dot(P, HT)
    R = np.ones((N,)) - np.einsum('ij,ji->i', np.dot(H, P), H.T)
    Y = np.dot(H, Bt)
    err = self._error(T, Y, R)

    penalty = err * 0.01 / L  # penalty is 1% of error at max(L)
    e[L] = err + L * penalty

    # MYOPT function
    # [iA  iB  iC  iD  iE] interval points,
    # halve the interval each time

    # initialize intervals
    iA = 1
    iE = L
    l = iE - iA
    iB = iA + l/4
    iC = iA + l/2
    iD = iA + 3*l/4

    l = 1000  # run the while loop at least once
    while l > 2:
        # calculate errors at points
        for idx in [iA, iB, iC, iD, iE]:
            if e[idx] == -1:  # skip already calculated errors
                rank1 = rank[:idx]
                # H1 = H[:, rank1]
                H1 = np.take(H, rank1, 1)
                HH1 = HH[rank1, :][:, rank1]
                HT1 = HT[rank1, :]

                P = np.linalg.inv(HH1)
                Bt1 = np.dot(P, HT1)
                R = np.ones((N,)) - np.einsum('ij,ji->i', np.dot(H1, P), H1.T)
                Y1 = np.dot(H1, Bt1)
                err = self._error(T, Y1, R)
                e[idx] = err + idx * penalty

        m = min(e[iA], e[iB], e[iC], e[iD], e[iE])  # find minimum element

        # halve the search interval
        if m in (e[iA], e[iB]):
            iE = iC
            iC = iB
        elif m in (e[iD], e[iE]):
            iA = iC
            iC = iD
        else:
            iA = iB
            iE = iD
        l = iE - iA
        iB = iA + l/4
        iD = iA + (3*l)/4

    k_opt = [n1 for n1 in [iA, iB, iC, iD, iE] if e[n1] == m][0]  # find minimum index
    best_L = rank[:k_opt]

    self.nnet._prune(best_L)
    self.nnet.add_batch(X, T)
    self.nnet.solve()
#    print "%d of %d neurons selected with a LOO validation" % (len(best_nn), nn)
#    if len(best_nn) > nn*0.9:
#        print "Hint: try re-training with more hidden neurons"





































# -*- coding: utf-8 -*-
"""Nvidia GPU-accelerated solver based on Scikit-CUDA, works without compiling anything.

GPU computations run in asyncronous mode: GPU is processing one batch of data while CPU prepares
the next batch. Loads GPU for 100% without waiting times, very fast and efficient. The requied
Scikit-CUDA is a single-line install in Linux: ``pip install scikit-cuda``. Tested on CUDA 7.

Created on Sat Sep 12 13:10:23 2015
@author: akusok
"""

from slfn import SLFN
import numpy as np
from scipy.linalg import lapack
from scipy.spatial.distance import cdist

from pycuda import gpuarray, cumath, autoinit
from pycuda.compiler import SourceModule
from skcuda import linalg, misc, cublas



class SLFNSkCUDA(SLFN):
    """Single Layer Feed-forward Network (SLFN) implementation on GPU with pyCUDA.

    To choose a specific GPU, use environmental variable ``CUDA_DEVICE``, for exampe
    ``CUDA_DEVICE=0 python myscript1.py & CUDA_DEVICE=1 python myscript2.py``.

    In single precision, only upper triangular part of HH matrix is computed to speedup the method.
    """

    def __init__(self, inputs, outputs, norm=None, precision=np.float64):
        super(SLFNSkCUDA, self).__init__(inputs, outputs, norm, precision)

        # startup GPU
        #self.ctx = misc.init_context(misc.init_device(nDevice))  # NO NO NO, crashes and does not release memory
        # use CUDA_DEVICE=0 python my-script.py
        try:
            linalg.init()
        except OSError as e:
            pass  # no 'cusolver' library which is paid and not needed
            # print "error initializing scikit-cuda: %s" % e
            # print "ignore if toolbox works"

        # precision-dependent stuff
        if precision is np.float64:
            self.posv = lapack.dposv
        else:
            self.posv = lapack.sposv
            self.handle = cublas.cublasCreate()

        # prepare GPU function kernels
        kernel = """
            __global__ void dev_sigm(%s *a) {
                unsigned idx = blockDim.x * blockIdx.x + threadIdx.x;
                a[idx] = 1.0 / ( exp(a[idx]) + 1 );
            }
            """
        kernel = kernel % "double" if self.precision is np.float64 else kernel % "float"
        self.dev_sigm = SourceModule(kernel).get_function("dev_sigm")
        self.dev_sigm.prepare("P")

        # GPU transformation functions
        self.func["lin"] = self._dev_lin
        self.func["sigm"] = self._dev_sigm
        self.func["tanh"] = self._dev_tanh
        self.func["rbf_l1"] = self._dev_rbfl1
        self.func["rbf_l2"] = self._dev_rbfl2
        self.func["rbf_linf"] = self._dev_rbflinf

    def _dev_lin(self, devX, devW, devB):
        """Linear function on GPU.

        Returns:
            devH (gpuarray): GPU matrix with the result.
        """
        devH = misc.add_matvec(linalg.dot(devX, devW), devB, axis=1)
        return devH

    def _dev_sigm(self, devX, devW, devB):
        """Compute Sigmoid on GPU for a given array and return array."""

#        def sigm(a):
#            block = a._block
#            grid = (int(np.ceil(1.0 * np.prod(a.shape) / block[0])), 1)
#            dev_sigm.prepared_call(grid, block, a.gpudata)
#            return a

        devH = misc.add_matvec(linalg.dot(devX, devW), devB, axis=1)
        block = devH._block
        grid = (int(np.ceil(1.0 * np.prod(devH.shape) / block[0])), 1)
        self.dev_sigm.prepared_call(grid, block, devH.gpudata)
        return devH

    def _dev_tanh(self, devX, devW, devB):
        """Hyperbolic tangent function on GPU.

        Returns:
            devH (gpuarray): GPU matrix with the result.
        """
        devH = misc.add_matvec(linalg.dot(devX, devW), devB, axis=1)
        cumath.tanh(devH, out=devH)
        return devH

    def _dev_rbfl1(self, devX, devW, devB):
        # TODO: make proper GPU implementation of RBF_L1
        X = devX.get()
        W = devW.get()
        B = devB.get()
        devH = gpuarray.to_gpu(np.exp(-cdist(X, W.T, "cityblock")**2 / B))
        return devH

    def _dev_rbfl2(self, devX, devW, devB):
        # TODO: make proper GPU implementation of RBF_L2
        X = devX.get()
        W = devW.get()
        B = devB.get()
        devH = gpuarray.to_gpu(np.exp(-cdist(X, W.T, "euclidean")**2 / B))
        return devH

    def _dev_rbflinf(self, devX, devW, devB):
        # TODO: make proper GPU implementation of RBF_Linf
        X = devX.get()
        W = devW.get()
        B = devB.get()
        devH = gpuarray.to_gpu(np.exp(-cdist(X, W.T, "chebyshev")**2 / B))
        return devH

    def add_neurons(self, number, func, W, B):
        """Add prepared neurons to the SLFN, merge with existing ones.

        Adds a number of specific neurons to SLFN network. Weights and biases
        must be provided for that function.

        If neurons of such type already exist, they are merged together.

        Args:
            number (int): the number of new neurons to add
            func (str): transformation function of hidden layer. Linear function creates a linear model.
            W (matrix): a 2-D matrix of neuron weights, size (`inputs` * `number`)
            B (vector): a 1-D vector of neuron biases, size (`number` * 1)
        """
        ntypes = [nr[1] for nr in self.neurons]  # existing types of neurons
        if func in ntypes:
            # add to an existing neuron type
            i = ntypes.index(func)
            nn0, _, devW, devB = self.neurons[i]
            number = nn0 + number
            devW = gpuarray.to_gpu(np.hstack((devW.get(), W)))
            devB = gpuarray.to_gpu(np.hstack((devB.get(), B)))
            self.neurons[i] = (number, func, devW, devB)
        else:
            # create a new neuron type
            devW = gpuarray.to_gpu(W)
            devB = gpuarray.to_gpu(B)
            self.neurons.append((number, func, devW, devB))
        self.reset()
        self.B = None

    def reset(self):
        """ Resets intermediate training results, releases memory that they use.

        Keeps solution of ELM, so a trained ELM remains operational.
        Can be called to free memory after an ELM is trained.
        """
        self.L = sum([n[0] for n in self.neurons])  # get number of neurons
        self.HH = None
        self.HT = None

    def _project(self, X, dev=False):
        """Projects X to H, an auxiliary function that implements a particular projection.

        For actual projection, use `ELM.project()` instead.

        Args:
            X (matrix): an input data matrix, size (N * `inputs`)
            dev (bool, optional): whether leave result in the GPU memory

        Returns:
            H (matrix): an SLFN hidden layer representation, size (N * `L`) where 'L' is number of neurons
        """
        assert self.neurons is not None, "ELM has no neurons"
        X = np.array(X, order="C", dtype=self.precision)
        devX = gpuarray.to_gpu(X)
        devH = gpuarray.empty((X.shape[0], self.L), dtype=self.precision)
        i = 0
        for nn, ftype, devW, devB in self.neurons:
            devH[:, i:i+nn] = self.func[ftype](devX, devW, devB)
            i += nn

        H = devH if dev else devH.get()
        return H

    def _predict(self, X, dev=False):
        """Predict a batch of data. Auxiliary function that implements a particular prediction.

        For prediction, use `ELM.predict()` instead.

        Args:
            X (matrix): input data size (N * `inputs`)
            dev (bool, optional): whether leave result in the GPU memory

        Returns:
            Y (matrix): predicted outputs size (N * `outputs`), always in float/double format.
        """
        assert self.B is not None, "Solve the task before predicting"
        devH = self._project(X, dev=True)
        devY = linalg.dot(devH, self.B)
        Y = devY if dev else devY.get()
        return Y

    def add_batch(self, X, T, wc=None):
        """Add a batch of training data to an iterative solution, weighted if neeed.

        The batch is processed as a whole, the training data is splitted in `ELM.add_data()` method.
        With parameters HH_out, HT_out, the output will be put into these matrices instead of model.

        Args:
            X (matrix): input data matrix size (N * `inputs`)
            T (matrix): output data matrix size (N * `outputs`)
            wc (vector): vector of weights for data samples, one weight per sample, size (N * 1)
            HH_out, HT_out (matrix, optional): output matrices to add batch result into, always given together
        """
        devH = self._project(X, dev=True)
        T = np.array(T, order="C", dtype=self.precision)
        devT = gpuarray.to_gpu(T)
        if wc is not None:  # apply weights if given
            w = np.array(wc**0.5, dtype=self.precision)[:, None]  # re-shape to column matrix
            devWC = gpuarray.to_gpu(w)
            misc.mult_matvec(devH, devWC, axis=0, out=devH)
            misc.mult_matvec(devT, devWC, axis=0, out=devT)

        if self.HH is None:  # initialize space for self.HH, self.HT
            self.HT = misc.zeros((self.L, self.outputs), dtype=self.precision)
            self.HH = linalg.eye(self.L, dtype=self.precision)
            self.HH *= self.norm

        linalg.add_dot(devH, devT, self.HT, transa='T')
        if self.precision is np.float64:
            linalg.add_dot(devH, devH, self.HH, transa='T')
        else:
            cublas.cublasSsyrk(self.handle, 'L', 'N', self.L, X.shape[0], 1, devH.ptr, self.L, 1, self.HH.ptr, self.L)
#        self.ctx.synchronize()  # GPU runs asyncronously without that

    def solve(self):
        """Compute output weights B, with fix for unstable solution.
        """
        HH = self.HH.get()
        HT = self.HT.get()
        B = self.solve_corr(HH, HT)
        self.B = gpuarray.to_gpu(B)

    def solve_corr(self, HH, HT):
        """Compute output weights B for given HH and HT.

        Simple but inefficient version, see a better one in solver_python.

        Args:
            HH (matrix): covariance matrix of hidden layer represenation H, size (`L` * `L`)
            HT (matrix): correlation matrix between H and outputs T, size (`L` * `outputs`)
        """
        _, B, info = self.posv(HH, HT)
        if info > 0:
            print "ELM covariance matrix is not full rank; solving with SVD (slow)"
            print "This happened because you have duplicated or too many neurons"
            HH = np.triu(HH) + np.triu(HH, k=1).T
            B = np.linalg.lstsq(HH, HT)[0]
        B = np.array(B, order='C', dtype=self.precision)
        return B

    def _prune(self, idx):
        """Leave only neurons with the given indexes.
        """
        idx = list(idx)
        neurons = []
        for k, func, devW, devB in self.neurons:
            ix1 = [i for i in idx if i < k]  # index for current neuron type
            idx = [i-k for i in idx if i >= k]
            number = len(ix1)
            W = devW.get()
            W = np.array(W[:, ix1], order='C')
            devW = gpuarray.to_gpu(W)
            B = devB.get()
            B = np.array(B[ix1], order='C')
            devB = gpuarray.to_gpu(B)
            neurons.append((number, func, devW, devB))
        self.neurons = neurons
        # reset invalid parameters
        self.reset()
        self.B = None

    def get_B(self):
        """Return B as a numpy array.
        """
        if self.B is None:
            B = None
        else:
            B = self.B.get()
        return B

    def set_B(self, B):
        """Set B as a numpy array.

        Args:
            B (matrix): output layer weights matrix, size (`L` * `outputs`)
        """
        assert B.shape[0] == self.L, "Incorrect first dimension: %d expected, %d found" % (self.L, B.shape[0])
        assert B.shape[1] == self.outputs, "Incorrect output dimension: %d expected, %d found" % (self.outputs, B.shape[1])
        self.B = gpuarray.to_gpu(B.astype(self.precision))

    def get_corr(self):
        """Return current correlation matrices.
        """
        if self.HH is None:
            HH = None
            HT = None
        else:
            HH = self.HH.get()
            HT = self.HT.get()
            HH = np.triu(HH) + np.triu(HH, k=1).T
        return HH, HT

    def set_corr(self, HH, HT):
        """Set pre-computed correlation matrices.

        Args:
            HH (matrix): covariance matrix of hidden layer represenation H, size (`L` * `L`)
            HT (matrix): correlation matrix between H and outputs T, size (`L` * `outputs`)
        """
        assert self.neurons is not None, "Add or load neurons before using ELM"
        assert HH.shape[0] == HH.shape[1], "HH must be a square matrix"
        msg = "Wrong HH dimension: (%d, %d) expected, %s found" % (self.L, self.L, HH.shape)
        assert HH.shape[0] == self.L, msg
        assert HH.shape[0] == HT.shape[0], "HH and HT must have the same number of rows (%d)" % self.L
        assert HT.shape[1] == self.outputs, "Number of columns in HT must equal number of outputs (%d)" % self.outputs
        self.HH = gpuarray.to_gpu(HH.astype(self.precision))
        self.HT = gpuarray.to_gpu(HT.astype(self.precision))

    def get_neurons(self):
        """Return current neurons.

        Returns:
            neurons (list of tuples (number/int, func/string, W/matrix, B/vector)): current neurons in the model
        """
        neurons = []
        for number, func, devW, devB in self.neurons:
            neurons.append((number, func, devW.get(), devB.get()))
        return neurons
















